
# ------------------------------------------------------------------------------
# Solve wolf-goat-cabbage problem
#
# Ralf Borndoerfer
# 12/2010
# ------------------------------------------------------------------------------

param T  := 7;

set Time := { 0..T };
set Pos  := { "Left", "Boat", "Right" };
set Item := { "Wolf", "Goat", "Cabbage" };

set Init := { 0, T } * { "Left", "Right" } * Item;

param v[Init] :=             | "Wolf", "Goat", "Cabbage" |
				| 0, "Left"  | 1,      1,      1         |
				| 0, "Right" | 0,      0,      0         |
				| T, "Left"  | 0,      0,      0         |
				| T, "Right" | 1,      1,      1         |;

# use this to solve the integer program
#var x[Time * Pos * Item] integers >= 0 <= 1;

# use this to solve the LP relaxation
var x[Time * Pos * Item] >= 0 <= 1;

minimize cost: sum <t,i> in Time * Item: x[t,"Left",i];

subto init: forall <t,p,i> in Init do
	x[t,p,i] == v[t,p,i];

subto flow_even_left: forall <t,i> in Time * Item with t < T and t mod 2 == 0 do
	x[t,"Left",i] == x[t+1,"Left",i] + x[t+1,"Boat",i];

subto flow_even_right: forall <t,i> in Time * Item with t < T and t mod 2 == 0 do
	x[t+1,"Right",i] == x[t,"Right",i] + x[t+1,"Boat",i];
	
subto flow_odd_right: forall <t,i> in Time * Item with t < T and t mod 2 == 1 do
	x[t,"Right",i] == x[t+1,"Right",i] + x[t+1,"Boat",i];
	
subto flow_odd_left: forall <t,i> in Time * Item with t < T and t mod 2 == 1 do
	x[t+1,"Left",i] == x[t,"Left",i] + x[t+1,"Boat",i];

subto boat: forall <t> in Time do
	sum <i> in Item: x[t,"Boat",i] <= 1;

subto safe_even_gc: forall <t> in Time with t mod 2 == 0 do
	- x[t,"Right","Wolf"] + x[t,"Right","Goat"] + x[t,"Right","Cabbage"] <= 1;

subto safe_even_wg: forall <t> in Time with t mod 2 == 0 do
	  x[t,"Right","Wolf"] + x[t,"Right","Goat"] - x[t,"Right","Cabbage"] <= 1;

subto safe_odd_wg: forall <t> in Time with t mod 2 == 1 do
	  x[t,"Left","Wolf"] + x[t,"Left","Goat"] <= 1;

subto safe_odd_gc: forall <t> in Time with t mod 2 == 1 do
	  x[t,"Left","Goat"] + x[t,"Left","Cabbage"] <= 1;
