#! /usr/bin/perl

#  ieq2lp.pl 1.0 - translate an "Alcuin-LP" in porta ieq-format to lp-format
#  Copyright (C) 2003 Ralf Borndoerfer (borndoerfer@zib.de)

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.

#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#  Zuse Institute Berlin (ZIB) hereby disclaims all copyright interest in the 
#  program `ieq2lp.pl' (which translates ieq-files) written by Ralf Borndoerfer.

#  Henry Thieme, Administrative Director of ZIB, 25 Februar 2003

use integer;

# Umrechnung Porta-Index j -> Alcuin-Index [xyz](t,i)
@id_x = ("x", "x", "x", "y", "y", "y", "z", "z", "z");
sub id_x { return $id_x[($_[0]-1)%9]; }
sub id_t { return ($_[0]-1)/9; }
sub id_i { return ($_[0]-1)%3 + 1; }
sub id_xti { 
    return join ``, id_x($_[0]), "(", id_t($_[0]), ",", id_i($_[0]), ")"; 
}

$debug = 0;

print $0, " version 1.0, Copyright (C) 2003 Ralf Borndoerfer\n",
    "\n",
    $0, " comes with ABSOLUTELY NO WARRANTY; for details see file `gpl.txt'.\n",
    "This is free software, and you are welcome to redistribute it\n",
    "under the GNU General Public License; see file `gpl.txt' for details.\n",
    "\n";

if (@ARGV != 1) { die "usage: ", $0, " <ieq-file>\n" };

$lpfile = $ARGV[0].".lp";
open SAVEOUT, ">&STDOUT";
open STDOUT, ">".$lpfile or die "Can't redirect stdout\n";

while (<>) {
    # eliminate line numbers
    s/(\(\ *\d+\ *\))//;

    # eliminate trailing spaces
    s/[ ]+$//;

    # add spaces around tokens
    s/(<=|>=|==|[=+-]|x\d+)/ $1 /g;

    # reduce spacing
    s/\ +/ /g;

    # reformat ==
    s/==/=/;

    if (/DIM *= *(\d+)/) {
        if ($debug) { print };

        print "Minimize\n";
        for ($j = 1; $j <= $1; $j++) { 
            print id_x($j) =~ /x/ ? " + " : " + 0 ", id_xti($j);
            print eval $j%9 == 0 ? "\n" : "";
        }
        print "\n";

        print "Subject to\n";
    }
    elsif (/=/) {
        if ($debug) { print };

        s/x(\d+)/&id_xti($1)/eg;

        print;
    }
    elsif (/END/) { 
        if ($debug) { print };

        print "End\n";
    }
}

close STDOUT;
open STDOUT, ">&SAVEOUT";
print "Output written to file ", $lpfile, "\n";

