#! /usr/bin/perl

#  lp2ieq.pl 1.0 - translate an "Alcuin-LP" in lp-format to porta ieq-format
#  Copyright (C) 2003 Ralf Borndoerfer (borndoerfer@zib.de)

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.

#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#  Zuse Institute Berlin (ZIB) hereby disclaims all copyright interest in the 
#  program `lp2ieq.pl' (which translates lp files) written by Ralf Borndoerfer.

#  Henry Thieme, Administrative Director of ZIB, 25 Februar 2003

use integer;

# Umrechnung Alcuin-Index [xyz](t,i) -> Porta-Index
%id_x = ("x" => 0, "y" => 1, "z" => 2);
sub num_j { 
    $_[0] =~ m/([xyz])\((\d+)\,(\d+)\)/;
    return 9*$2 + 3*$id_x{$1} + $3;
}
sub id_j { return join ``, "x", num_j($_[0]); }

$debug = 0;

$dim = 0;

print $0, " version 1.0, Copyright (C) 2003 Ralf Borndoerfer\n",
    "\n",
    $0, " comes with ABSOLUTELY NO WARRANTY; for details see file `gpl.txt'.\n",
    "This is free software, and you are welcome to redistribute it\n",
    "under the GNU General Public License; see file `gpl.txt' for details.\n",
    "\n";

if (@ARGV != 1) { die "usage: ", $0, " <lp-file>\n" };

$ieqfile = $ARGV[0].".ieq";
open SAVEOUT, ">&STDOUT";
open STDOUT, ">".$ieqfile or die "Can't redirect stdout\n";

while (<>) {
    # eliminate trailing spaces
    s/[ ]+$//;

    # add spaces around tokens
    s/(<=|>=|[=+-]|[xyz]\(\d+\,\d+\))/ $1 /g;

    # reduce spacing
    s/\ +/ /g;

    # reformat ==
    s/ = / == /;

    if (/=/) {
        if ($debug) { print };

        s/[xyz]\(\d+\,\d+\)/&id_j($&)/eg;

        print;
    }
    elsif (/Subject/) {
        if ($debug) { print };

        print "DIM = ", $dim, "\n\n";

        print "VALID\n";
        print "1 1 1  0 0 0  0 0 0   ";
        print "1 0 1  0 1 0  0 1 0   ";
        print "1 0 1  0 0 0  0 1 0   ";
        print "0 0 1  1 0 0  1 1 0   ";
        print "0 1 1  0 1 0  1 0 0   ";
        print "0 1 0  0 0 1  1 0 1   ";
        print "0 1 0  0 0 0  1 0 1   ";
        print "0 0 0  0 1 0  1 1 1   ";
        for ($t = 8; $t < $dim/9; $t += 2) {
            print "0 0 0  0 0 0  1 1 1   ";
            print "0 0 0  0 0 0  1 1 1   ";
        }
        print "\n\n";

        print "INEQUALITIES_SECTION\n";
    }
    elsif (/End/) { 
        if ($debug) { print };

        print "END\n";
    }
    else {
        if ($debug) { print };
        while (m/[xyz]\(\d+\,\d+\)/g) { 
            $j = num_j($&);
            if ($dim < $j) { $dim = $j; }
            if ($debug) { print " [", $&, ",", $j, "]"; }
        }
        if ($debug) { print "\n"; }
    }
}

close STDOUT;
open STDOUT, ">&SAVEOUT";
print "Output written to file ", $ieqfile, "\n";
